"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsentFactoryContract = void 0;
const objects_1 = require("../interfaces/objects");
const abi_1 = require("../interfaces/objects/abi");
const objects_2 = require("@snickerdoodlelabs/objects");
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
let ConsentFactoryContract = class ConsentFactoryContract {
    constructor(providerOrSigner, consentFactoryAddress) {
        this.providerOrSigner = providerOrSigner;
        this.contract = new ethers_1.ethers.Contract(consentFactoryAddress, abi_1.ContractsAbis.ConsentFactoryAbi.abi, providerOrSigner);
    }
    getContractAddress() {
        var _a;
        return (0, objects_2.EVMContractAddress)(((_a = this.contract) === null || _a === void 0 ? void 0 : _a.address) || "");
    }
    // Function to help user create consent
    // After creating consent, call getUserDeployedConsentsCount to get total number of deployed consents
    createConsent(ownerAddress, baseUri, name) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.createConsent(ownerAddress, baseUri, name), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call createConsent()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentFactoryContractError("Wait for optIn() failed", "Unknown", e);
            });
        })
            .andThen((receipt) => {
            // Get the hash of the event
            const event = "ConsentDeployed(address,address)";
            const eventHash = ethers_1.ethers.utils.keccak256(ethers_1.ethers.utils.toUtf8Bytes(event));
            // Filter out for the ConsentDeployed event from the receipt's logs
            // returns an array
            const consentDeployedLog = receipt.logs.filter((_log) => _log.topics[0] == eventHash);
            // access the data and topics from the filtered log
            const data = consentDeployedLog[0].data;
            const topics = consentDeployedLog[0].topics;
            // Declare a new interface
            const Interface = ethers_1.ethers.utils.Interface;
            const iface = new Interface([
                "event ConsentDeployed(address indexed owner, address indexed consentAddress)",
            ]);
            // Decode the log from the given data and topic
            const decodedLog = iface.decodeEventLog("ConsentDeployed", data, topics);
            const deployedConsentAddress = decodedLog.consentAddress;
            return (0, neverthrow_1.okAsync)(deployedConsentAddress);
        });
    }
    // Gets the count of user's deployed Consents
    getUserDeployedConsentsCount(ownerAddress) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getUserDeployedConsentsCount(ownerAddress), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call getUserDeployedConsentsCount()", e.reason, e);
        }).map((count) => {
            return count.toNumber();
        });
    }
    // Gets the array of user deployed Consents by index count
    // Index values can be anywhere between the count obtained from getUserDeployedConsentsCount
    // eg. If user has [0x123, 0xabc, 0x456] Consent contracts, query with startingIndex 0 and endingIndex 2 to get full list
    getUserDeployedConsentsByIndex(ownerAddress, startingIndex, endingIndex) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getUserDeployedConsentsByIndex(ownerAddress, startingIndex, endingIndex), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call getUserDeployedConsentsByIndex()", e.reason, e);
        });
    }
    // get the latest deployed consent address by owner account address
    getUserDeployedConsents(ownerAddress) {
        return this.getUserDeployedConsentsCount(ownerAddress).andThen((count) => {
            return this.getUserDeployedConsentsByIndex(ownerAddress, 0, count);
        });
    }
    // Gets the count of Consent addresses user has specific roles for
    getUserRoleAddressesCount(ownerAddress, role) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getUserConsentAddressesCount(ownerAddress, role), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call getUserRoleAddressesCount()", e.reason, e);
        }).map((count) => {
            return count.toNumber();
        });
    }
    // Gets the array of Consent addresses user has specific roles for
    // Index values can be anywhere between the count obtained from getUserRoleAddressesCount
    // eg. If user has [0x123, 0xabc, 0x456] Consent contracts, query with startingIndex 0 and endingIndex 2 to get full list
    getUserRoleAddressesCountByIndex(ownerAddress, role, startingIndex, endingIndex) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getUserRoleAddressesCountByIndex(ownerAddress, role, startingIndex, endingIndex), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call filters.getUserRoleAddressesCountByIndex()", e.reason, e);
        });
    }
    getDeployedConsents() {
        const eventFilter = this.contract.filters.ConsentDeployed();
        return neverthrow_1.ResultAsync.fromPromise(this.contract.queryFilter(eventFilter), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call contract.queryFilter() for ConsentDeployed event", e.reason, e);
        }).map((events) => {
            const consents = [];
            events.forEach((event) => {
                var _a;
                if ((_a = event === null || event === void 0 ? void 0 : event.args) === null || _a === void 0 ? void 0 : _a.consentAddress) {
                    consents.push((0, objects_2.EVMContractAddress)(event.args.consentAddress));
                }
            });
            return consents;
        });
    }
    // Marketplace functions
    getMaxTagsPerListing() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.maxTagsPerListing(), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call getMaxTagsPerListing()", e.reason, e);
        }).map((num) => {
            return num.toNumber();
        });
    }
    getNumberOfListings(tag) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getTagTotal(tag), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call getNumberOfListings()", e.reason, e);
        }).map((num) => {
            return num.toNumber();
        });
    }
    getListingDuration() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getListingDuration(), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call getListingDuration()", e.reason, e);
        }).map((num) => {
            return num.toNumber();
        });
    }
    setListingDuration(listingDuration) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.setListingDuration(listingDuration), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call setListingDuration()", e.reason, e);
        });
    }
    setMaxTagsPerListing(maxTagsPerListing) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.setMaxTagsPerListing(maxTagsPerListing), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call setListingDuration()", e.reason, e);
        });
    }
    initializeTag(tag, newHead) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.initializeTag(tag, newHead), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call initializeTag()", e.reason, e);
        });
    }
    insertUpstream(tag, newSlot, existingSlot) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.insertUpstream(tag, newSlot, existingSlot), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call insertUpstream()", e.reason, e);
        }).map((tx) => {
            return new objects_1.WrappedTransactionResponse(tx);
        });
    }
    insertDownstream(tag, existingSlot, newSlot) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.insertUpstream(tag, existingSlot, newSlot), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call insertDownstream()", e.reason, e);
        }).map((tx) => {
            return new objects_1.WrappedTransactionResponse(tx);
        });
    }
    replaceExpiredListing(tag, slot) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.replaceExpiredListing(tag, slot), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call insertDownstream()", e.reason, e);
        }).map((tx) => {
            return new objects_1.WrappedTransactionResponse(tx);
        });
    }
    removeListing(tag, removedSlot) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.replaceExpiredListing(tag, removedSlot), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call removeListing()", e.reason, e);
        }).map((tx) => {
            return new objects_1.WrappedTransactionResponse(tx);
        });
    }
    adminRemoveListing(tag, removedSlot) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.adminRemoveListing(tag, removedSlot), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call adminRemoveListing()", e.reason, e);
        }).map((tx) => {
            return new objects_1.WrappedTransactionResponse(tx);
        });
    }
    getListingDetail(tag, slot) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getListing(tag, slot), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call getListing()", e.reason, e);
        }).map((listing) => {
            var _a;
            return new objects_2.MarketplaceListing((0, objects_2.BigNumberString)(listing.previous.toString()), (0, objects_2.BigNumberString)(listing.next.toString()), listing.consentContract, (0, objects_2.UnixTimestamp)((_a = listing.timeExpiring) === null || _a === void 0 ? void 0 : _a.toNumber()), (0, objects_2.IpfsCID)(""), // TODO: Update contract to also return its CID for getListing (only does this with getListingsForward/backward atm)
            slot, tag);
        });
    }
    getListingsForward(tag, startingSlot, numberOfSlots, filterActive) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getListingsForward(tag, startingSlot, numberOfSlots, filterActive), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call getListingsForward()", e.reason, e);
        }).map(([cids, listings]) => {
            return listings.map((listing, index) => {
                return new objects_2.MarketplaceListing((0, objects_2.BigNumberString)(listing.previous.toString()), (0, objects_2.BigNumberString)(listing.next.toString()), listing.consentContract, (0, objects_2.UnixTimestamp)(listing.timeExpiring.toNumber()), (0, objects_2.IpfsCID)(cids[index]), (0, objects_2.BigNumberString)(listings[index + 1].previous.toString()), tag);
            });
        });
    }
    getListingsBackward(tag, startingSlot, numberOfSlots, filterActive) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getListingsForward(tag, startingSlot, numberOfSlots, filterActive), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call getListingsForward()", e.reason, e);
        }).map(([cids, listings]) => {
            return listings.map((listing, index) => {
                var _a;
                return new objects_2.MarketplaceListing((0, objects_2.BigNumberString)(listing.previous.toString()), (0, objects_2.BigNumberString)(listing.next.toString()), listing.consentContract, (0, objects_2.UnixTimestamp)((_a = listing.timeExpiring) === null || _a === void 0 ? void 0 : _a.toNumber()), (0, objects_2.IpfsCID)(cids[index]), (0, objects_2.BigNumberString)(listings[index + 1].previous.toString()), tag);
            });
        });
    }
    getTagTotal(tag) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getTagTotal(tag), (e) => {
            return new objects_2.ConsentFactoryContractError("Unable to call getTagTotal()", e.reason, e);
        }).map((count) => {
            return count.toNumber();
        });
    }
};
ConsentFactoryContract = __decorate([
    (0, inversify_1.injectable)()
], ConsentFactoryContract);
exports.ConsentFactoryContract = ConsentFactoryContract;
// I listingStruct { at the place where we're using it, and don't have to export here
// Alternative option is to get the deployed Consent addresses through filtering event ConsentDeployed() event
/* // TODO: Replace Promise<any> with correct types returned from ConsentDeployed() and queryFilter()
  public getConsentsDeployedByOwner(
    ownerAddress: EVMAccountAddress,
  ): ResultAsync<EVMContractAddress[], ConsentFactoryContractError> {
    return ResultAsync.fromPromise(
      this.contract.filters.ConsentDeployed(ownerAddress) as Promise<any>,
      (e) => {
        return new ConsentFactoryContractError(
          "Unable to call filters.ConsentDeployed()",
          (e as IBlockchainError).reason,
          e,
        );
      },
    )
      .andThen((_logs) => {
        return ResultAsync.fromPromise(
          this.contract.queryFilter(_logs) as Promise<any>,
          (e) => {
            return new ConsentFactoryContractError(
              "Unable to call filters.ConsentDeployed()",
              (e as IBlockchainError).reason,
              e,
            );
          },
        );
      })
      .map((logs) => {
        return logs.map((log) => log.args.consentAddress);
      });
  }
} */
//# sourceMappingURL=ConsentFactoryContract.js.map